//#########################################################
//#
//# Titre : 	Utilitaires Liste Chainee et BD LINUX Automne 08
//#				SIF-1015 - Système d'exploitation
//#				Université du Québec à Trois-Rivières
//#
//# Auteur : 	Francois Meunier et David Laperriere
//#	Date :		Septembre 2008
//#
//# Langage : 	ANSI C on LINUX 
//#
//#######################################

#include "gestionListeChaineeBD.h"

//Pointeur de tête de liste
extern struct record* head;
//Pointeur de queue de liste pour ajout rapide
extern struct record* queue;

//#######################################
//#
//# Recherche un item dans la liste chaînée
//#
//# RETOUR:	Un pointeur vers l'item recherché
//# 		
//# 		Retourne NULL dans le cas où l'item
//#			est introuvable
//#
struct record * findItem(const int no){


	//La liste est vide puisqu'il n'y a que l'item bidon
	if (head==queue) return NULL;

	//Pointeur de navigation
	struct record * ptr = head;

	//Tant qu'un item suivant existe
	while (ptr->suivant!=NULL){

		//Déplacement du pointeur de navigation
		ptr=ptr->suivant;

		//Est-ce l'item recherché?
		if (ptr->vente.nosequentiel==no){
			//On retourne un pointeur sur l'item (toujours vérrouillé)
			return ptr;
			}
		}

	//On retourne un pointeur NULL
	return NULL;
	}

//#######################################
//#
//# Recherche le PRÉDÉCESSEUR d'un item dans la liste chaînée
//#
//# RETOUR:	Le pointeur vers le prédécesseur est retourné
//# 		
//#			
//# 		Retourne NULL dans le cas où l'item est introuvable
//#
struct record * findPrev(const int no){

	//La liste est vide puis qu'il n'u a que l'item bidon
	if (head==queue) return NULL;

	//Pointeur de navigation
	struct record * ptr = head;

	//Tant qu'un item suivant existe
	while (ptr->suivant!=NULL){

		//Est-ce le prédécesseur de l'item recherché?
		if (ptr->suivant->vente.nosequentiel==no){
			//On retourne un pointeur sur l'item précédent
			return ptr;
		}

		//Déplacement du pointeur de navigation
		ptr=ptr->suivant;
		}

	//On retourne un pointeur NULL
	return NULL;
	}

//#######################################
//#
//# Ajoute un item à la fin de la liste chaînée
//#
void addItem(const char* nv, const char* np, const int nbv){

	//Création de l'enregistrement en mémoire
	struct record* ni = (struct record*)malloc(sizeof(struct record));

	//Affectation des valeurs des champs
	ni->vente.nosequentiel	= ((queue==NULL)?0:queue->vente.nosequentiel+1);
	strcpy(ni->vente.nomvendeur, nv);
	strcpy(ni->vente.noproduit, np);
	ni->vente.nombrevendu	= nbv;
	ni->suivant				= NULL;

	//Vérrouillage de l'item en queue de liste (si requis)
	if (queue!=NULL) {
		//Ajout "réel" de l'item dans la liste chaînée
		queue->suivant=ni;
		}

	//Ajustement des pointeurs de tête et de queue
	queue = ni;
	if(head==NULL) head = ni;
}

//#######################################
//#
//# Retire un item de la liste chaînée
//#
void removeItem(const int no){

	//Vérification sommaire (no>0 et liste non vide)
	//Le numéro ZERO est interdit puis qu'il représente
	//  le numéro séquentiel de l'item bidon
	if ((no<1)||(head==queue))
		return;

	//Pointeur de recherche
	struct record * ptr = findPrev(no);

	//L'item a été trouvé
	if (ptr!=NULL){

		//Mémorisation du pointeur de l'item en cours de suppression
		struct record * tptr = ptr->suivant;

		//Réajustement du pointeur de queue	(si requis)
		if (queue==ptr->suivant) queue=ptr;

		//Réajustement des pointeur de la liste chaînée
		ptr->suivant = ptr->suivant->suivant;

		//Libération de la zone de mémoire
		free(tptr);
		}
	}

//#######################################
//#
//# Modifie un item de la liste chaînée
//#
void modifyItem(const int no, const char* nv, const char* np, const int nbv){

	//Vérification sommaire (no>0 et liste non vide)
	//Le numéro ZERO est interdit puis qu'il représente
	//  le numéro séquentiel de l'item bidon
	if ((no<1)||(head==queue))
		return;

	//Recherche de l'élément à modifier
	struct record * ptr = findItem(no);

	//L'élément à été trouvé
	if (ptr!=NULL){

		//Modification des champs de l'élément trouvé
		strcpy(ptr->vente.nomvendeur, nv);
		strcpy(ptr->vente.noproduit, np);
		ptr->vente.nombrevendu	= nbv;
		}
	}

//#######################################
//#
//# Affiche les items dont le numéro séquentiel est compris dans une plage
//#
void listItems(const int start, const int end){

	//Affichage des entêtes de colonnes
	printf("no  vendeur                        produit     nb\n");
	printf("=== ============================== =========== =====\n");

	struct record * optr = head;			//dummy element
	struct record * ptr = head->suivant;	//first element


	while (ptr!=NULL){

		//L'item a un numéro séquentiel dans l'interval défini
		if ((ptr->vente.nosequentiel>=start)&&(ptr->vente.nosequentiel<=end)){
			printf("%3d %-30s %-11s %5d\n",
				ptr->vente.nosequentiel,
				ptr->vente.nomvendeur,
				ptr->vente.noproduit,
				ptr->vente.nombrevendu
				);
			}

		optr=ptr;
		if (ptr->vente.nosequentiel>end){
			//L'ensemble des items potentiels sont maintenant passés
			//Déplacement immédiatement à la FIN de la liste
			//Notez que le pointeur optr est toujours valide
			ptr=NULL;
			}
		else{
			ptr = ptr->suivant;
			}

		}

	//Affichage des pieds de colonnes
	printf("=== ============================== =========== =====\n\n");
	}

